<?php // 投稿ページ・固定ページですべてのH2タグの前に広告を表示する独自ウィジェットエリアを追加
// H2タグの前に広告を表示する回数（-1で制限なしに）
define('LIMIT_H2_AD', 10);


// 独自ウィジェットエリアを作成
register_sidebar(array(
  'name' => '独自ウィジェットエリア（投稿本文中）',
  'id' => 'widget-multi-h2',
  'description' => '投稿ページ・固定ページのすべてのH2タグ（最大'.LIMIT_H2_AD.'回まで）の前に広告を表示。',
  'before_widget' => '<div class="widget widget-multi-h2 %2$s">',
  'after_widget' => '</div>',
  'before_title' => '',
  'after_title' => '',
));


// H2タグを判別する正規表現を定数化
if (!defined('H2_REG'))
  define('H2_REG', apply_filters('insertion_heading_regexp', '/<h2/i'));


// 本文中にH2タグが最初に含まれている箇所を返す（含まれない場合はnullを返す）
if(!function_exists('get_h2_included_in_body')):
function get_h2_included_in_body($the_content){
  //H2見出しが本文中にあるかどうか
  if(preg_match(H2_REG, $the_content, $h2results)){
    return $h2results[0];
  }
}
endif;


// すべてのH2タグ手前にウィジェットを挿入（LIMIT_H2_AD定数で回数制限）
if(!function_exists('add_ads_before_multi_h2')):
function add_ads_before_multi_h2($the_content) {
  if(is_singular()  // 投稿ページ・固定ページのとき
    // && is_all_adsenses_visible() // AdSense設定項目で表示が許可されているか
  ){
    // ウィジェットの中身を取ってきて$ad_templateの中にセットする
    ob_start();  //バッファ開始
    dynamic_sidebar('widget-multi-h2');  // 独自ウィジェットエリアの中身をバッファに出力
    $ad_template = ob_get_clean();  // バッファを出力＆終了

    // 本文にH2タグが含まれていれば取得
    $h2result = get_h2_included_in_body($the_content);  
    // H2見出しが本文中にある場合のみ
    if($h2result){
      // 1つ目のH2タグはいったんH99タグに置換（1つ目のH2タグ手前には広告を設置しない）
      $the_content = preg_replace('/<h2/i', '<h99', $the_content, 1);
      // 各H2タグの前に広告を挿入（LIMIT_H2_ADで回数を制限）
      $the_content = preg_replace(H2_REG, $ad_template.PHP_EOL.PHP_EOL.$h2result, $the_content, LIMIT_H2_AD);
      // H99をH2に戻す
      $the_content = preg_replace('/<h99/i', '<h2', $the_content, 1);
    }
  }
  return $the_content;
}
endif;
add_filter('the_content','add_ads_before_multi_h2');